from dolfin import *
import numpy, sys
import scipy.io
set_log_level(WARNING)

r = int(sys.argv[1])
tau = 1.0
gamma = 10.0*r*r
uexact = Expression("(x[0]-pow(x[0],2))*exp(3*x[0])*sin(2*pi*x[1])")
f = Expression("-exp(3*x[0])*(-9*pow(x[0],2)-3*x[0]+4)*sin(2*pi*x[1]) \
				+(x[0]-pow(x[0],2))*exp(3*x[0])*4*pow(pi,2)*sin(2*pi*x[1])")
g = Constant(0.0)

N = 6
L2error = []
ENerror = []
n1 = 4

for i in range(0, N):
	
	# generate mesh, function space and print info
	Th = UnitSquareMesh(n1,n1)
	Vh = FunctionSpace(Th,"DG",r)

	# Define variational problem
	u = TrialFunction(Vh)
	v = TestFunction(Vh)
	n = FacetNormal(Th)
	h = CellSize(Th)

	a = dot(grad(u), grad(v))*dx - dot(jump(v, n), avg(grad(u)))*dS \
		- tau*dot(jump(u, n), avg(grad(v)))*dS + gamma/avg(h)*dot(jump(u, n), jump(v, n))*dS \
		- v*dot(grad(u), n)*ds - tau*u*dot(grad(v), n)*ds + gamma/h*u*v*ds
	L = f*v*dx - tau*g*dot(grad(v), n)*ds + gamma/h*g*v*ds

	# Compute solution
	u = Function(Vh)
	solve(a==L, u)
	
	# Error L2
	zero = Expression("0.0")
	L2error.append(errornorm(uexact, u)/errornorm(zero,u))
	
	# Error energy norm
	ue = interpolate(uexact,Vh)
	ene = dot(grad(u-ue),grad(u-ue))*dx + gamma/avg(h)*dot(jump(u-ue,n),jump(u-ue,n))*dS \
	    + gamma/h*(u-ue)*(u-ue)*ds
	enu = dot(grad(u),grad(u))*dx + gamma/avg(h)*dot(jump(u,n),jump(u,n))*dS \
	    + gamma/h*(u)*(u)*ds
	ENerror.append(sqrt(assemble(ene))/sqrt(assemble(enu)))
	
	# Update n1
	n1 *= 2

print "\nr = ", r

from math import log as ln
print "\nExpected order: ", r+1
for i in range(0, N-1):
    print "L2 order:", ln(L2error[i]/L2error[i+1])/ln(2.)
print "\nExpected order: ", r
for i in range(0, N-1):
    print "EN order:", ln(ENerror[i]/ENerror[i+1])/ln(2.)

# Save to .mat
# scipy.io.savemat("errors.mat", {"l2e": L2error.array(), "ene": ENerror.array()})
