from dolfin import *
import numpy, sys
from math import log as ln
set_log_level(WARNING)

# start the main code
degree = int(sys.argv[2])
L2error = []
H1error = []

n  = int(sys.argv[1])
g  = Expression("-pow(2*n+1,2)*pow(pow(x[0],2)+pow(x[1],2),n-0.5)", n=n)
ue = Expression("pow(sqrt(pow(x[0],2)+pow(x[1],2)),2*n+1)"        , n=n)  # in H^(2*n+1)
tol = 1E-12

# define boundary conditions
def boundary13(x, on_boundary):
	return on_boundary and (abs(x[1]+1) < tol or abs(x[1]-1) < tol)
def boundary24(x, on_boundary):
	return on_boundary and (abs(x[0]+1) < tol or abs(x[0]-1) < tol)

u13 = Expression("pow(sqrt(pow(x[0],2)+1),2*n+1)",n=n)
u24 = Expression("pow(sqrt(pow(x[1],2)+1),2*n+1)",n=n)

n1 = 4
for i in range(0, 6):
    
	# generate mesh, function space and print info
	Th = RectangleMesh(-1,-1,1,1,n1,n1)
	Vh = FunctionSpace(Th,"CG",degree)
	u_ = Function(Vh)
	
	print "\nnumber of sites: ", Th.num_vertices()
	print "number of triangles: ", Th.num_cells()
	print "degree of freedom: ", u_.vector().array().size
		
	# boundary conditions
	bc = [DirichletBC(Vh, u13, boundary13), DirichletBC(Vh, u24, boundary24)]

	# Define variational problem
	u = TrialFunction(Vh)
	v = TestFunction(Vh)
	a = inner(grad(u), grad(v))*dx
	L = g*v*dx
	
	# Compute solution
	solve(a == L, u_, bc)

	# Error
	L2 = errornorm(ue, u_, norm_type="L2")
	H10 = errornorm(ue, u_, norm_type="H10")
	H1error.append(L2+H10)   # order s   -- s = min(r,2n)
	L2error.append(L2)   	 # order s+1 -- s = min(r,2n)
	
	# Increase points
	n1 *= 2

print "\nn = ", n
print "degree = ", degree

print "\nExpected order", min(degree,2*n)
for i in range(0, 5):
    print "H1 order:", ln(H1error[i]/H1error[i+1])/ln(2.)
print "\nExpected order", min(degree,2*n)+1
for i in range(0, 5):
    print "L2 order:", ln(L2error[i]/L2error[i+1])/ln(2.)
